@testset "ImpulseResponse" begin
    df = exampledata(:gk)
    ns = (:logcpi, :logip, :ff, :ebp, :ff4_tc)
    r = lp(df, :ebp, wnames=ns, nlag=12, nhorz=48, vce=HRVCE())
    f = irf(r, :ebp, :ff4_tc, lag=1)

    @test sprint(show, f) == "ImpulseResponse"
    w = VERSION < v"1.4.0" ? " " : ""
    @test sprint(show, MIME("text/plain"), f, context=:displaysize=>(15,80)) == """
        ImpulseResponse with 48 horizons:
        ────────────────────────────────────────────────────────────────
             Estimate  Std. Error      z  Pr(>|z|)  Lower 90%  Upper 90%
        ────────────────────────────────────────────────────────────────
        0   0.394494     0.473835   0.83    0.4061  -0.388576    1.17757
        1   0.161872     0.509504   0.32    0.7510  -0.680168    1.00391
        2  -0.0203338    0.611236  -0.03    0.9735  -1.03053     0.98986
        3   0.152802     0.724653   0.21    0.8332  -1.04487     1.35047
        4   0.147288     0.737092   0.20    0.8418  -1.07097     1.36554
        5   1.0421       0.771451   1.35    0.1783  -0.232978    2.31717
        ────────────────────────────────────────────────────────────────
        42 rows of estimates omitted"""

    r1 = lp(df, :ebp, wnames=ns, nlag=12, minhorz=1, nhorz=5, vce=HRVCE())
    f1 = irf(r1, :ebp, :ff4_tc, lag=1)
    @test sprint(show, MIME("text/plain"), f1) == """
        ImpulseResponse with 5 horizons:
        ────────────────────────────────────────────────────────────────
             Estimate  Std. Error      z  Pr(>|z|)  Lower 90%  Upper 90%
        ────────────────────────────────────────────────────────────────
        1   0.161872     0.509504   0.32    0.7510  -0.680168    1.00391
        2  -0.0203338    0.611236  -0.03    0.9735  -1.03053     0.98986
        3   0.152802     0.724653   0.21    0.8332  -1.04487     1.35047
        4   0.147288     0.737092   0.20    0.8418  -1.07097     1.36554
        5   1.0421       0.771451   1.35    0.1783  -0.232978    2.31717
        ────────────────────────────────────────────────────────────────"""

    @test sprint(show, MIME("text/plain"), coeftable(f1, level=0.95)) == """
        ────────────────────────────────────────────────────────────────
             Estimate  Std. Error      z  Pr(>|z|)  Lower 95%  Upper 95%
        ────────────────────────────────────────────────────────────────
        1   0.161872     0.509504   0.32    0.7510  -0.842942    1.16669
        2  -0.0203338    0.611236  -0.03    0.9735  -1.22582     1.18515
        3   0.152802     0.724653   0.21    0.8332  -1.27641     1.58201
        4   0.147288     0.737092   0.20    0.8418  -1.3065      1.60108
        5   1.0421       0.771451   1.35    0.1783  -0.479509    2.5637$w
        ────────────────────────────────────────────────────────────────"""

    est = SmoothLP(:ff4_tc, search=grid(10), criterion=GCV())
    r2 = lp(est, df, :ebp, xnames=:ff4_tc, wnames=ns, nlag=12, minhorz=1, nhorz=5, vce=HARVCE(EWC()))
    f2 = irf(r2, :ebp, :ff4_tc)
    @test sprint(show, MIME("text/plain"), f2) == """
        ImpulseResponse with 5 horizons:
        ─────────────────────────────────────────────────────────────
           Estimate  Std. Error     z  Pr(>|z|)  Lower 90%  Upper 90%
        ─────────────────────────────────────────────────────────────
        1  0.657581    0.233319  2.82    0.0124   0.250233    1.06493
        2  0.870131    0.239008  3.64    0.0022   0.452852    1.28741
        3  0.948191    0.307555  3.08    0.0071   0.411237    1.48515
        4  0.892463    0.286437  3.12    0.0067   0.392378    1.39255
        5  0.703113    0.494902  1.42    0.1746  -0.160928    1.56715
        ─────────────────────────────────────────────────────────────"""
end
